@@ -282,7 +282,7 @@ def length(style, name, value, font_size=None, pixels_only=False):
     elif unit in LENGTHS_TO_PIXELS:
         # Convert absolute lengths to pixels
         result = value.value * LENGTHS_TO_PIXELS[unit]
-    elif unit in ('em', 'ex', 'ch', 'rem'):
+    elif unit in ('em', 'ex', 'ch', 'rem', 'lh', 'rlh'):
         if font_size is None:
             font_size = style['font_size']
         if unit == 'ex':
@@ -296,6 +296,12 @@ def length(style, name, value, font_size=None, pixels_only=False):
             result = value.value * font_size
         elif unit == 'rem':
             result = value.value * style.root_style['font_size']
+        elif unit == 'lh':
+            line_height, _ = strut_layout(style)
+            result = value.value * line_height
+        elif unit == 'rlh':
+            line_height, _ = strut_layout(style.root_style)
+            result = value.value * line_height
     else:
         # A percentage or 'auto': no conversion needed.
         return value
@@ -337,7 +343,7 @@ def image_orientation(style, name, values):
     if values in ('none', 'from-image'):
         return values
     angle, flip = values
-    return (int(round(angle / pi * 2)) % 4 * 90, flip)
+    return (round(angle / pi * 2) % 4 * 90, flip)
 
 
 @register_computer('border-top-width')
@@ -478,7 +484,7 @@ def _content_list(style, values):
                 if attr is None:
                     computed_value = None
                 else:
-                    computed_value = (value[0], ((attr,) + value[1][1:]))
+                    computed_value = (value[0], (attr, *value[1][1:]))
             else:
                 computed_value = value
         if computed_value is None:
