@@ -68,9 +68,11 @@ class Box:
     # Default, may be overriden on instances.
     is_table_wrapper = False
     is_flex_item = False
+    is_grid_item = False
     is_for_root_element = False
     is_column = False
     is_leader = False
+    is_outside_marker = False
 
     # Other properties
     transformation_matrix = None
@@ -79,16 +81,28 @@ class Box:
     footnote = None
     cached_counter_values = None
     missing_link = None
+    link_annotation = None
+    force_fragmentation = False
 
     # Default, overriden on some subclasses
     def all_children(self):
-        return ()
+        return self.children
+
+    def descendants(self, placeholders=False):
+        """A flat generator for a box, its children and descendants."""
+        yield self
+        for child in self.children:
+            if placeholders or isinstance(child, Box):
+                yield from child.descendants(placeholders)
+            else:
+                yield child
 
     def __init__(self, element_tag, style, element):
         self.element_tag = element_tag
         self.element = element
         self.style = style
         self.remove_decoration_sides = set()
+        self.children = []
 
     def __repr__(self):
         return f'<{type(self).__name__} {self.element_tag}>'
@@ -289,6 +303,27 @@ class Box:
             self.is_floated() or self.is_absolutely_positioned() or
             self.is_running() or self.is_footnote())
 
+    def is_monolithic(self):
+        """Return whether this box is monolithic."""
+        # https://www.w3.org/TR/css-break-3/#monolithic
+        return (
+            isinstance(self, AtomicInlineLevelBox) or
+            isinstance(self, ReplacedBox) or
+            self.style['overflow'] in ('auto', 'scroll') or
+            (self.style['overflow'] == 'hidden' and
+             self.style['height'] != 'auto'))
+
+    def establishes_formatting_context(self):
+        """Return whether this box establishes a block formatting context."""
+        # See https://www.w3.org/TR/CSS2/visuren.html#block-formatting
+        return (
+            self.is_floated() or
+            self.is_absolutely_positioned() or
+            self.is_column or
+            (isinstance(self, BlockContainerBox) and not isinstance(self, BlockBox)) or
+            (isinstance(self, BlockBox) and self.style['overflow'] != 'visible') or
+            'flow-root' in self.style['display'])
+
     # Start and end page values for named pages
 
     def page_values(self):
@@ -315,6 +350,12 @@ class Box:
                 return not isinstance(self, (LineBox, TextBox))
         return False
 
+    def is_form(self):
+        """Return whether this box is a form element."""
+        if self.element is None:
+            return False
+        return self.element.tag == 'form'
+
 
 class ParentBox(Box):
     """A box that has children."""
@@ -322,9 +363,6 @@ class ParentBox(Box):
         super().__init__(element_tag, style, element)
         self.children = tuple(children)
 
-    def all_children(self):
-        return self.children
-
     def _reset_spacing(self, side):
         """Set to 0 the margin, padding and border of ``side``."""
         self.remove_decoration_sides.add(side)
@@ -343,7 +381,7 @@ class ParentBox(Box):
     def copy_with_children(self, new_children):
         """Create a new equivalent box with given ``new_children``."""
         new_box = self.copy()
-        new_box.children = list(new_children)
+        new_box.children = new_children
 
         # Clear and reset removed decorations as we don't want to keep the
         # previous data, for example when a box is split between two pages.
@@ -353,19 +391,9 @@ class ParentBox(Box):
 
     def deepcopy(self):
         result = self.copy()
-        result.children = tuple(child.deepcopy() for child in self.children)
+        result.children = list(child.deepcopy() for child in self.children)
         return result
 
-    def descendants(self):
-        """A flat generator for a box, its children and descendants."""
-        yield self
-        for child in self.children:
-            if isinstance(child, ParentBox):
-                for grand_child in child.descendants():
-                    yield grand_child
-            else:
-                yield child
-
     def get_wrapped_table(self):
         """Get the table wrapped by the box."""
         assert self.is_table_wrapper
@@ -377,13 +405,18 @@ class ParentBox(Box):
 
     def page_values(self):
         start_value, end_value = super().page_values()
-        if self.children:
-            if len(self.children) == 1:
-                page_values = self.children[0].page_values()
+        # TODO: We should find Class A possible page breaks according to
+        # https://drafts.csswg.org/css-page-3/#propdef-page
+        # Keep only children in normal flow for now.
+        children = [
+            child for child in self.children if child.is_in_normal_flow()]
+        if children:
+            if len(children) == 1:
+                page_values = children[0].page_values()
                 start_value = page_values[0] or start_value
                 end_value = page_values[1] or end_value
             else:
-                start_box, end_box = self.children[0], self.children[-1]
+                start_box, end_box = children[0], children[-1]
                 start_value = start_box.page_values()[0] or start_value
                 end_value = end_box.page_values()[1] or end_value
         return start_value, end_value
@@ -472,8 +505,6 @@ class InlineBox(InlineLevelBox, ParentBox):
     inline box.
 
     """
-    link_annotation = None
-
     def hit_area(self):
         """Return the (x, y, w, h) rectangle where the box is clickable."""
         # Use line-height (margin_height) rather than border_height
@@ -601,9 +632,6 @@ class TableColumnGroupBox(ParentBox):
     internal_table_or_caption = True
     proper_parents = (TableBox, InlineTableBox)
 
-    # Default value. May be overriden on instances.
-    span = 1
-
     # Columns groups never have margins or paddings
     margin_top = 0
     margin_bottom = 0
@@ -638,9 +666,6 @@ class TableColumnBox(ParentBox):
     internal_table_or_caption = True
     proper_parents = (TableBox, InlineTableBox, TableColumnGroupBox)
 
-    # Default value. May be overriden on instances.
-    span = 1
-
     # Columns never have margins or paddings
     margin_top = 0
     margin_bottom = 0
@@ -713,6 +738,19 @@ class PageBox(ParentBox):
     def __repr__(self):
         return f'<{type(self).__name__} {self.page_type}>'
 
+    @property
+    def bleed(self):
+        return {
+            side: self.style[f'bleed_{side}'].value
+            for side in ('top', 'right', 'bottom', 'left')}
+
+    @property
+    def bleed_area(self):
+        return (
+            -self.bleed['left'], -self.bleed['top'],
+            self.margin_width() + self.bleed['left'] + self.bleed['right'],
+            self.margin_height() + self.bleed['top'] + self.bleed['bottom'])
+
 
 class MarginBox(BlockContainerBox):
     """Box in page margins, as defined in CSS3 Paged Media"""
@@ -756,3 +794,23 @@ class InlineFlexBox(FlexContainerBox, InlineLevelBox):
     It behaves as inline on the outside and as a flex container on the inside.
 
     """
+
+
+class GridContainerBox(ParentBox):
+    """A box that contains only grid-items."""
+
+
+class GridBox(GridContainerBox, BlockLevelBox):
+    """A box that is both block-level and a grid container.
+
+    It behaves as block on the outside and as a grid container on the inside.
+
+    """
+
+
+class InlineGridBox(GridContainerBox, InlineLevelBox):
+    """A box that is both inline-level and a grid container.
+
+    It behaves as inline on the outside and as a grid container on the inside.
+
+    """
