@@ -1,6 +1,7 @@
 """Fonts integration in PDF."""
 
 import io
+import re
 from hashlib import md5
 from logging import WARNING
 from math import ceil
@@ -34,7 +35,7 @@ class Font:
                 part.split('=')[0]: float(part.split('=')[1])
                 for part in ffi.string(variations).decode().split(',')}
         if weight := self.variations.get('weight'):
-            self.weight = int(round(weight))
+            self.weight = round(weight)
             pango.pango_font_description_set_weight(description, weight)
         else:
             self.weight = pango.pango_font_description_get_weight(description)
@@ -58,24 +59,18 @@ class Font:
             in md5(description_string, usedforsecurity=False).digest()[:6])
 
         # Set font name.
-        fields = description_string.split(b' ')
-        if fields and b'=' in fields[-1]:
-            fields.pop()  # Remove variations
-        if fields:
-            fields.pop()  # Remove font size
-        else:
-            fields = [b'Unknown']
-        self.name = b'/' + self.hash.encode() + b'+' + b'-'.join(fields)
+        name = re.split(b' [#@]', description_string)[0]
+        self.name = b'/' + self.hash.encode() + b'+' + name.replace(b' ', b'-')
 
         # Set ascent and descent.
         if self.font_size:
             pango_metrics = pango.pango_font_get_metrics(pango_font, ffi.NULL)
-            self.ascent = int(round(
+            self.ascent = round(
                 pango.pango_font_metrics_get_ascent(pango_metrics) * FROM_UNITS /
-                self.font_size * 1000))
-            self.descent = -int(round(
+                self.font_size * 1000)
+            self.descent = -round(
                 pango.pango_font_metrics_get_descent(pango_metrics) * FROM_UNITS /
-                self.font_size * 1000))
+                self.font_size * 1000)
         else:
             self.ascent = self.descent = 0
 
@@ -112,7 +107,7 @@ class Font:
         self.flags = 2 ** (3 - 1)  # Symbolic, custom character set
         if self.style:
             self.flags += 2 ** (7 - 1)  # Italic
-        if b'Serif' in fields:
+        if b'Serif' in name.split(b' '):
             self.flags += 2 ** (2 - 1)  # Serif
 
     def clean(self, cmap, hinting):
@@ -342,15 +337,19 @@ def build_fonts_dictionary(pdf, fonts, compress, subset, options):
             b'/CMapType 2 def',
             b'1 begincodespacerange',
             b'<0000> <ffff>',
-            b'endcodespacerange',
-            f'{len(cmap)} beginbfchar'.encode()], compress=compress)
-        for glyph, text in cmap.items():
-            unicode_codepoints = ''.join(
-                f'{letter.encode("utf-16-be").hex()}' for letter in text)
-            to_unicode.stream.append(
-                f'<{glyph:04x}> <{unicode_codepoints}>'.encode())
+            b'endcodespacerange'], compress=compress)
+        cmap_length = len(cmap)
+        cmap_items = tuple(cmap.items())
+        for i in range(ceil(cmap_length / 100)):
+            batch_length = min(100, cmap_length - i * 100)
+            to_unicode.stream.append(f'{batch_length} beginbfchar'.encode())
+            for glyph, text in cmap_items[i*100:(i+1)*100]:
+                unicode_codepoints = ''.join(
+                    f'{letter.encode("utf-16-be").hex()}' for letter in text)
+                to_unicode.stream.append(
+                    f'<{glyph:04x}> <{unicode_codepoints}>'.encode())
+            to_unicode.stream.append(b'endbfchar')
         to_unicode.stream.extend([
-            b'endbfchar',
             b'endcmap',
             b'CMapName currentdict /CMap defineresource pop',
             b'end',
@@ -561,7 +560,7 @@ def _build_vector_font_dictionary(font_dictionary, pdf, font, widths, compress,
     })
     if str(pdf_version) <= '1.4':  # Cast for bytes and None
         cids = sorted(font.widths)
-        padded_width = int(ceil((cids[-1] + 1) / 8))
+        padded_width = ceil((cids[-1] + 1) / 8)
         bits = ['0'] * padded_width * 8
         for cid in cids:
             bits[cid] = '1'
