@@ -208,14 +208,23 @@ class FontConfiguration:
             match = SubElement(root, 'match', target='font')
             test = SubElement(match, 'test', name='file', compare='eq')
             SubElement(test, 'string').text = str(font_path)
-            edit = SubElement(match, 'edit', name='fontfeatures', mode=mode)
             descriptors = {
                 rules[0][0].replace('-', '_'): rules[0][1] for rules in
                 rule_descriptors.get('font_variant', [])}
             settings = rule_descriptors.get('font_feature_settings', 'normal')
             features = font_features(font_feature_settings=settings, **descriptors)
-            for key, value in features.items():
-                SubElement(edit, 'string').text = f'{key} {value}'
+            if features:
+                edit = SubElement(match, 'edit', name='fontfeatures', mode=mode)
+                for key, value in features.items():
+                    SubElement(edit, 'string').text = f'{key} {value}'
+            if unicode_ranges := rule_descriptors.get('unicode_range'):
+                edit = SubElement(match, 'edit', name='charset', mode=mode)
+                plus = SubElement(edit, 'plus')
+                for unicode_range in unicode_ranges:
+                    charset = SubElement(plus, 'charset')
+                    range_ = SubElement(charset, 'range')
+                    for value in (unicode_range.start, unicode_range.end):
+                        SubElement(range_, 'int').text = f'0x{value:x}'
             header = (
                 b'<?xml version="1.0"?>',
                 b'<!DOCTYPE fontconfig SYSTEM "urn:fontconfig:fonts.dtd">')
@@ -365,7 +374,7 @@ def get_pango_font_key(pango_font):
     # TODO: This value is stable for a given Pango font in a given Pango map, but can’t
     # be cached with just the Pango font as a key because two Pango fonts could point to
     # the same address for two different Pango maps. We should cache it in the
-    # FontConfiguration object. See https://github.com/Kozea/WeasyPrint/issues/2144
+    # FontConfiguration object. See issue #2144.
     description = ffi.gc(
         pango.pango_font_describe(pango_font), pango.pango_font_description_free)
     font_size = pango.pango_font_description_get_size(description) * FROM_UNITS
