@@ -215,6 +215,12 @@ def layout_document(html, root_box, context, max_loops=8):
         yield page
 
 
+class FakeList(list):
+    """List in which you can’t append objects."""
+    def append(self, item):
+        pass
+
+
 class LayoutContext:
     def __init__(self, style_for, get_image_from_uri, font_config,
                  counter_style, target_collector):
@@ -271,6 +277,17 @@ class LayoutContext:
         else:
             self.excluded_shapes = None
 
+    def create_flex_formatting_context(self):
+        self.excluded_shapes = FakeList()
+        self._excluded_shapes_lists.append(self.excluded_shapes)
+
+    def finish_flex_formatting_context(self, root_box):
+        self._excluded_shapes_lists.pop()
+        if self._excluded_shapes_lists:
+            self.excluded_shapes = self._excluded_shapes_lists[-1]
+        else:
+            self.excluded_shapes = None
+
     def get_string_set_for(self, page, name, keyword='first'):
         """Resolve value of string function."""
         return self.get_string_or_element_for(
@@ -372,10 +389,13 @@ class LayoutContext:
             if not self.in_column:
                 self.page_bottom -= footnote_area.margin_height()
             last_child = footnote_area.children[-1]
-            overflow = (
-                last_child.position_y + last_child.margin_height() >
+            last_child_bottom = (
+                last_child.position_y + last_child.margin_height() -
+                last_child.margin_bottom)
+            footnote_area_bottom = (
                 footnote_area.position_y + footnote_area.margin_height() -
                 footnote_area.margin_bottom)
+            overflow = last_child_bottom > footnote_area_bottom
             return overflow
         else:
             self.current_footnote_area.height = 0
