@@ -10,7 +10,7 @@ import zlib
 from gzip import GzipFile
 from pathlib import Path
 from urllib.parse import quote, unquote, urljoin, urlsplit
-from urllib.request import Request, pathname2url, urlopen
+from urllib.request import Request, pathname2url, url2pathname, urlopen
 
 from . import __version__
 from .logger import LOGGER
@@ -177,7 +177,7 @@ def ensure_url(string):
     return string if url_is_absolute(string) else path2url(string)
 
 
-def default_url_fetcher(url, timeout=10, ssl_context=None):
+def default_url_fetcher(url, timeout=10, ssl_context=None, http_headers=None):
     """Fetch an external resource such as an image or stylesheet.
 
     Another callable with the same signature can be given as the
@@ -190,6 +190,8 @@ def default_url_fetcher(url, timeout=10, ssl_context=None):
         The number of seconds before HTTP requests are dropped.
     :param ssl.SSLContext ssl_context:
         An SSL context used for HTTP requests.
+    :param dict http_headers:
+        Additional HTTP headers used for HTTP requests.
     :raises: An exception indicating failure, e.g. :obj:`ValueError` on
         syntactically invalid URL.
     :returns: A :obj:`dict` with the following keys:
@@ -205,7 +207,9 @@ def default_url_fetcher(url, timeout=10, ssl_context=None):
           if there were e.g. HTTP redirects.
         * Optionally: ``filename``, the filename of the resource. Usually
           derived from the *filename* parameter in a *Content-Disposition*
-          header
+          header.
+        * Optionally: ``path``, the path of the resource if it is stored on the
+          local filesystem.
 
         If a ``file_obj`` key is given, it is the caller’s responsibility
         to call ``file_obj.close()``. The default function used internally to
@@ -218,19 +222,26 @@ def default_url_fetcher(url, timeout=10, ssl_context=None):
         # See https://bugs.python.org/issue34702
         if url.startswith('file://'):
             url = url.split('?')[0]
+            path = url2pathname(url.removeprefix('file:'))
+        else:
+            path = None
 
         url = iri_to_uri(url)
+        if http_headers is not None:
+            http_headers = {**HTTP_HEADERS, **http_headers}
+        else:
+            http_headers = HTTP_HEADERS
         response = urlopen(
-            Request(url, headers=HTTP_HEADERS), timeout=timeout,
+            Request(url, headers=http_headers), timeout=timeout,
             context=ssl_context)
-        response_info = response.info()
         result = {
-            'redirected_url': response.geturl(),
-            'mime_type': response_info.get_content_type(),
-            'encoding': response_info.get_param('charset'),
-            'filename': response_info.get_filename(),
+            'redirected_url': response.url,
+            'mime_type': response.headers.get_content_type(),
+            'encoding': response.headers.get_param('charset'),
+            'filename': response.headers.get_filename(),
+            'path': path,
         }
-        content_encoding = response_info.get('Content-Encoding')
+        content_encoding = response.headers.get('Content-Encoding')
         if content_encoding == 'gzip':
             result['file_obj'] = StreamingGzipFile(fileobj=response)
         elif content_encoding == 'deflate':
